; Copyright (C) Internet Systems Consortium, Inc. ("ISC")
;
; SPDX-License-Identifier: MPL-2.0
;
; This Source Code Form is subject to the terms of the Mozilla Public
; License, v. 2.0.  If a copy of the MPL was not distributed with this
; file, you can obtain one at https://mozilla.org/MPL/2.0/.
;
; See the COPYRIGHT file distributed with this work for additional
; information regarding copyright ownership.

{% raw -%}
@ 0 SOA . . 0 0 0 0 0
@ 0 NS .
@ 0 A 10.53.0.4
{% endraw -%}

{% for i in range(10000) -%}
x@i@ 0 in a 10.53.0.1
{% endfor %}