/*
 * Copyright (C) Internet Systems Consortium, Inc. ("ISC")
 *
 * SPDX-License-Identifier: MPL-2.0
 *
 * This Source Code Form is subject to the terms of the Mozilla Public
 * License, v. 2.0.  If a copy of the MPL was not distributed with this
 * file, you can obtain one at https://mozilla.org/MPL/2.0/.
 *
 * See the COPYRIGHT file distributed with this work for additional
 * information regarding copyright ownership.
 */

options {
	query-source address 10.53.0.1;
	notify-source 10.53.0.1;
	transfer-source 10.53.0.1;
	port @PORT@;
	pid-file "named.pid";
	listen-on { 10.53.0.1; };
	listen-on-v6 { none; };
	dnssec-validation no;
	recursion no;
	notify yes;
};

logging {
	channel default_log {
	  buffered no;
	  file "@TMPDIR@/example.log" versions 1 size 1k suffix increment; # small size
	  severity debug 100;
	  print-time yes;
	};
	category default { default_log; default_debug; };
	category lame-servers { null; };

	channel query_log {
	  file "query_log";
	  print-time yes;
	  buffered yes;
	};
	category queries { query_log; };
};

controls {
	inet 10.53.0.1 port @CONTROLPORT@ allow { any; } keys { "rndc-key"; };
};

key rndc-key {
	secret "1234abcd8765";
	algorithm hmac-sha256;
};
